package de.jottyfan.timetrack.modules.done;

import static de.jottyfan.timetrack.db.done.Tables.V_DAY;
import static de.jottyfan.timetrack.db.profile.Tables.T_LOGIN;

import java.sql.Time;
import java.time.Duration;
import java.time.LocalDate;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jooq.DSLContext;
import org.jooq.Field;
import org.jooq.Record5;
import org.jooq.SelectConditionStep;
import org.jooq.impl.DSL;
import org.jooq.types.YearToSecond;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import de.jottyfan.timetrack.modules.done.model.DaysumBean;

/**
 * 
 * @author jotty
 * 
 */
@Repository
public class DoneRepository {
	private static final Logger LOGGER = LogManager.getLogger(DoneRepository.class);

	@Autowired
	private DSLContext jooq;

	public DaysumBean getDaysum(LocalDate day, String login) {
		Field<LocalTime> WORKTIME = DSL.field("worktime", LocalTime.class);
		Field<LocalTime> BREAKTIME = DSL.field("breaktime", LocalTime.class);
		
		SelectConditionStep<Record5<LocalTime, LocalTime, LocalTime, LocalTime, YearToSecond>> sql = jooq
		// @formatter:off
			.select(V_DAY.STARTTIME,
					    V_DAY.ENDTIME,
					    V_DAY.WORKTIME.cast(LocalTime.class).as(WORKTIME),
					    V_DAY.BREAKTIME.cast(LocalTime.class).as(BREAKTIME),
					    V_DAY.DAY_OVERTIME)
			.from(V_DAY)
			.innerJoin(T_LOGIN).on(T_LOGIN.PK.eq(V_DAY.FK_LOGIN))
			.where(V_DAY.DAY.eq(day))
			.and(T_LOGIN.LOGIN.eq(login));
		// @formatter:on
		LOGGER.trace(sql);
		Record5<LocalTime, LocalTime, LocalTime, LocalTime, YearToSecond> r = sql.fetchOne();
		if (r == null) {
			return null;
		} else {
			DaysumBean bean = new DaysumBean();
			bean.setDaytimeFrom(r.get(V_DAY.STARTTIME));
			bean.setDaytimeUntil(r.get(V_DAY.ENDTIME));
			bean.setDayworktime(r.get(WORKTIME));
			bean.setBreaks(r.get(BREAKTIME));
			YearToSecond dayOvertime = r.get(V_DAY.DAY_OVERTIME);
			Duration dayOvertimeDuration = dayOvertime == null ? null : dayOvertime.toDuration();
			String dayOvertimeString = dayOvertimeDuration == null ? null : String.format("%3d:%2d", dayOvertimeDuration.toHours(), Math.abs(dayOvertimeDuration.toMinutes() % 60));
			bean.setDayOvertime(dayOvertimeString == null ? "?" : dayOvertimeString);
			bean.setTotalOvertime(getOvertimeOf(day, login));
			return bean;
		}
	}
	
	private LocalTime getOvertimeOf(LocalDate day, String login) {
		// using sql string here, because the DSL.sum function does not allow LocalTime, whilst the postgreSQL sum function does allow time calculations
		String sqlRaw = """
				select o.worktime_offset + sum(d.worktime - r.required) as overtime
				from done.v_day d
				inner join done.t_required_worktime r on r.day = d.day and r.fk_login = d.fk_login 
				inner join done.t_overtime o on o.fk_login = d.fk_login
				inner join profile.t_login p on p.pk = d.fk_login 
				where o.impact < d.day
				and d.day <= '?'
				and p.login = '?'
				group by d.fk_login, o.worktime_offset;
				""";
		String sql = sqlRaw.replaceFirst("\\?", day.format(DateTimeFormatter.ISO_DATE)).replaceFirst("\\?", login);
		LOGGER.trace(sql);
		Time time = (Time) jooq.fetchOne(sql).get(0);
		return time.toLocalTime();
	}
}
